from datetime import datetime as dt

__title__ = 'helga'
__version__ = '1.6.8'
__author__ = 'Shaun Duncan'
__license__ = 'MIT/GPLv3'
__copyright__ = 'Copyright {0} Shaun Duncan'.format(dt.now().strftime('%Y'))
__description__ = 'A full-featured IRC bot for Python/Twisted with plugin support'
