<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <title>{{ title }}</title>
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.1/css/bootstrap.min.css">
        <style>
            pre {
                background: none;
                border: none;
                font-size: 12px;
                padding: 0px;
                white-space: pre-wrap;
                white-space: -moz-pre-wrap;
                white-space: -pre-wrap;
                white-space: -o-pre-wrap;
                word-wrap: break-word;
                word-break: normal;
                margin: 0px;
            }

            td {
                border: none;
                padding: 0px;
            }

            table {
                margin-top: 25px;
            }
        </style>
    </head>
    <body>
        <div class="container">
            <div class="page-header">
                <h1>{{ title }}</h1>
            </div>
            <p><b>All Times In UTC</b></p>
