#CHANNEL_LOGGING = True
#CHANNEL_LOGGING_DIR = '/tmp/helga_channel_logs'
#CHANNEL_LOGGING_HIDE_CHANNELS = ['#foo']
#WEBHOOKS_CREDENTIALS = [('sduncan', 'sduncan')]
#ENABLED_WEBHOOKS = ['logger']
#COMMAND_ARGS_SHLEX = True

#SERVER = {
#    'HOST': 'chat.hipchat.com',
#    'PORT': 5222,
#    'JID': '291857_1819242@chat.hipchat.com',
#    'PASSWORD': 'One19@3#2',
#    'MUC_HOST': 'conf.hipchat.com',
#    'TYPE': 'xmpp',
#    'MANAGE_ROSTER': True,
#}
#CHANNELS = [
#    ('#291857_helgabot-dev',),
#]
#NICK = 'Helga Bot'
#COMMAND_PREFIX_BOTNICK = '@?HelgaBot'
#HIPCHAT_API_TOKEN = 'sIdbGFT7X608xZDhn0VNAdKrqwuOefg7DXdR1cdJ'

#SERVER = {
#    'HOST': 'helga.local',
#    'PORT': 5222,
#    'USERNAME': 'helga',
#    'PASSWORD': 'helga',
#    'TYPE': 'xmpp',
#}

SERVER = {
    'HOST': 'hipchat.pdrop.net',
    'PORT': 5222,
    'JID': '1_11@chat.btf.hipchat.com',
    'PASSWORD': 'One19@3#2',
    'MUC_HOST': 'conf.btf.hipchat.com',
    'TYPE': 'xmpp',
}
CHANNELS = [
    ('#1_xcmg',),
]
NICK = 'Shaun Duncan'
COMMAND_PREFIX_BOTNICK = '@?ShaunDuncan'
HIPCHAT_API_TOKEN = 'sK7EBKb7exckFIXBZ89BJ13lCrRXPtTAaagPG0PD'
HIPCHAT_API_ENDPOINT = 'https://hipchat.pdrop.net/api'
