{{> header }}
<p>
    <a href="/logger/{{ channel_name }}/{{ date }}.txt" class="btn btn-primary">Download Log</a>
</p>

<table class="table table-bordered table-striped">
    <tbody>
        {{# messages }}
            <tr>
                <td><a href="#{{ time }}" name="{{ time }}">{{ time }}</a></td>
                <td>{{ nick }}</td>
                <td><pre>{{ message }}</pre></td>
            </tr>
        {{/ messages }}
    </tbody>
</table>
{{> footer }}

